@@ -7,6 +7,7 @@ package specutil
 import (
 	"errors"
 	"fmt"
+	"slices"
 	"strconv"
 	"strings"
 
@@ -61,6 +62,8 @@ type (
 		Proc  func(*sqlspec.Func) (*schema.Proc, error)
 		// Triggers add themselves to the relevant tables/views.
 		Triggers func(*schema.Realm, []*sqlspec.Trigger) error
+		// Objects add themselves to the realm.
+		Objects func(*schema.Realm) error
 	}
 
 	// SchemaFuncs represents a set of spec functions
@@ -71,6 +74,20 @@ type (
 		Func  func(*schema.Func) (*sqlspec.Func, error)
 		Proc  func(*schema.Proc) (*sqlspec.Func, error)
 	}
+	// RefNamer is an interface for objects that can
+	// return their reference.
+	RefNamer interface {
+		// Ref returns the reference to the object.
+		Ref() *schemahcl.Ref
+	}
+	// SpecTypeNamer is an interface for objects that can
+	// return their spec type and name.
+	SpecTypeNamer interface {
+		// SpecType returns the spec type of the object.
+		SpecType() string
+		// SpecName returns the spec name of the object.
+		SpecName() string
+	}
 )
 
 const (
@@ -243,6 +260,11 @@ func Scan(r *schema.Realm, doc *ScanDoc, funcs *ScanFuncs) error {
 			return err
 		}
 	}
+	if funcs.Objects != nil {
+		if err := funcs.Objects(r); err != nil {
+			return err
+		}
+	}
 	for o, refs := range deps {
 		var err error
 		switch o := o.(type) {
@@ -722,6 +744,10 @@ func dependsOn(realm *schema.Realm, objects []schema.Object) (*schemahcl.Attr, b
 			n, s = d.Name, d.Schema.Name
 		case *schema.Proc:
 			n, s = d.Name, d.Schema.Name
+		case RefNamer:
+			// If the object is a reference, add it to the depends_on list.
+			deps = append(deps, d.Ref())
+			continue
 		}
 		if names[name(o, n)] > 1 {
 			path = append(path, s)
@@ -731,6 +757,9 @@ func dependsOn(realm *schema.Realm, objects []schema.Object) (*schemahcl.Attr, b
 		}))
 	}
 	if len(deps) > 0 {
+		slices.SortFunc(deps, func(l, r *schemahcl.Ref) int {
+			return strings.Compare(l.V, r.V)
+		})
 		return schemahcl.RefsAttr("depends_on", deps...), true
 	}
 	return nil, false
@@ -771,6 +800,15 @@ func fromDependsOn[T interface{ AddDeps(...schema.Object) T }](loc string, t T,
 			o, err = findT(ns, q, n, func(s *schema.Schema, name string) (*schema.Proc, bool) {
 				return s.Proc(name)
 			})
+		default:
+			o, err = findT(ns, q, n, func(s *schema.Schema, name string) (schema.Object, bool) {
+				return s.Object(func(o schema.Object) bool {
+					if o, ok := o.(SpecTypeNamer); ok {
+						return p[0].T == o.SpecType() && name == o.SpecName()
+					}
+					return false
+				})
+			})
 		}
 		if err != nil {
 			return fmt.Errorf("find %s refrence for %s.depends_on[%d]: %w", loc, p[0].T, i, err)
