@@ -5,6 +5,7 @@
 package specutil
 
 import (
+	"math"
 	"testing"
 
 	"ariga.io/atlas/schemahcl"
@@ -12,6 +13,7 @@ import (
 	"ariga.io/atlas/sql/sqlspec"
 
 	"github.com/stretchr/testify/require"
+	"github.com/zclconf/go-cty/cty"
 )
 
 func TestFromSpec_SchemaName(t *testing.T) {
@@ -22,7 +24,7 @@ func TestFromSpec_SchemaName(t *testing.T) {
 		},
 	}
 	sc.Tables[0].Schema = sc
-	spec, err := FromSchema(sc, &Funcs{
+	spec, err := FromSchema(sc, &SchemaFuncs{
 		Table: func(*schema.Table) (*sqlspec.Table, error) {
 			return &sqlspec.Table{}, nil
 		},
@@ -94,3 +96,52 @@ func TestFromForeignKey(t *testing.T) {
 		},
 	}, key)
 }
+
+func TestDefault(t *testing.T) {
+	for _, tt := range []struct {
+		v cty.Value
+		x string
+	}{
+		{
+			v: cty.NumberUIntVal(1),
+			x: "1",
+		},
+		{
+			v: cty.NumberIntVal(1),
+			x: "1",
+		},
+		{
+			v: cty.NumberFloatVal(1),
+			x: "1",
+		},
+		{
+			v: cty.NumberIntVal(-100),
+			x: "-100",
+		},
+		{
+			v: cty.NumberFloatVal(-100),
+			x: "-100",
+		},
+		{
+			v: cty.NumberUIntVal(math.MaxUint64),
+			x: "18446744073709551615",
+		},
+		{
+			v: cty.NumberIntVal(math.MinInt64),
+			x: "-9223372036854775808",
+		},
+		{
+			v: cty.NumberFloatVal(-1024.1024),
+			x: "-1024.1024",
+		},
+	} {
+		// From cty.Value (HCL) to database literal.
+		x, err := Default(tt.v)
+		require.NoError(t, err)
+		require.Equal(t, tt.x, x.(*schema.Literal).V)
+		// From database literal to cty.Value (HCL).
+		v, err := ColumnDefault(schema.NewColumn("").SetDefault(&schema.Literal{V: tt.x}))
+		require.NoError(t, err)
+		require.True(t, tt.v.Equals(v).True())
+	}
+}
