@@ -6,6 +6,7 @@ package specutil
 
 import (
 	"fmt"
+	"slices"
 
 	"ariga.io/atlas/schemahcl"
 	"ariga.io/atlas/sql/schema"
@@ -243,6 +244,24 @@ func QualifyReferences(tableSpecs []*sqlspec.Table, realm *schema.Realm) error {
 	return nil
 }
 
+// ObjectRef returns a reference to the object. In case there is more than one object of
+// this type with the same name, the reference will be qualified with the schema name.
+func ObjectRef(s *schema.Schema, o SpecTypeNamer) *schemahcl.Ref {
+	typ, name := o.SpecType(), o.SpecName()
+	idx := schemahcl.PathIndex{T: typ, V: []string{name}}
+	if s != nil && s.Realm != nil && len(s.Realm.Schemas) > 1 && slices.ContainsFunc(s.Realm.Schemas, func(s1 *schema.Schema) bool {
+		return s1 != s && slices.ContainsFunc(s1.Objects, func(o1 schema.Object) bool {
+			if e, ok := o1.(SpecTypeNamer); ok {
+				return e.SpecType() == typ && e.SpecName() == name
+			}
+			return false
+		})
+	}) {
+		idx.V = append([]string{s.Name}, idx.V...)
+	}
+	return schemahcl.BuildRef([]schemahcl.PathIndex{idx})
+}
+
 // HCLBytesFunc returns a helper that evaluates an HCL document from a byte slice instead
 // of from an hclparse.Parser instance.
 func HCLBytesFunc(ev schemahcl.Evaluator) func(b []byte, v any, inp map[string]cty.Value) error {
