@@ -224,7 +224,8 @@ func ValuesEqual(v1, v2 []string) bool {
 // ModeInspectSchema returns the InspectMode or its default.
 func ModeInspectSchema(o *schema.InspectOptions) schema.InspectMode {
 	if o == nil || o.Mode == 0 {
-		return schema.InspectSchemas | schema.InspectTables | schema.InspectViews | schema.InspectFuncs | schema.InspectTypes
+		return schema.InspectSchemas | schema.InspectTables | schema.InspectViews | schema.InspectFuncs |
+			schema.InspectTypes | schema.InspectObjects | schema.InspectTriggers
 	}
 	return o.Mode
 }
@@ -232,7 +233,8 @@ func ModeInspectSchema(o *schema.InspectOptions) schema.InspectMode {
 // ModeInspectRealm returns the InspectMode or its default.
 func ModeInspectRealm(o *schema.InspectRealmOption) schema.InspectMode {
 	if o == nil || o.Mode == 0 {
-		return schema.InspectSchemas | schema.InspectTables | schema.InspectViews | schema.InspectFuncs | schema.InspectTypes
+		return schema.InspectSchemas | schema.InspectTables | schema.InspectViews | schema.InspectFuncs |
+			schema.InspectTypes | schema.InspectObjects | schema.InspectTriggers
 	}
 	return o.Mode
 }
@@ -265,6 +267,11 @@ func (b *Builder) P(phrases ...string) *Builder {
 	return b
 }
 
+// Int64 writes the given value to the builder in base 10.
+func (b *Builder) Int64(v int64) *Builder {
+	return b.P(strconv.FormatInt(v, 10))
+}
+
 // Ident writes the given string quoted as an SQL identifier.
 func (b *Builder) Ident(s string) *Builder {
 	if s != "" {
@@ -288,24 +295,50 @@ func (b *Builder) Table(t *schema.Table) *Builder {
 	return b.mayQualify(t.Schema, t.Name)
 }
 
+// TableColumn writes the table's resource identifier to the builder, prefixed
+// with the schema name if exists.
+func (b *Builder) TableColumn(t *schema.Table, c *schema.Column) *Builder {
+	return b.mayQualify(t.Schema, t.Name, c.Name)
+}
+
 // Func writes the function identifier to the builder, prefixed
 // with the schema name if exists.
 func (b *Builder) Func(f *schema.Func) *Builder {
 	return b.mayQualify(f.Schema, f.Name)
 }
 
+// FuncCall writes the function identifier to the builder as a function call,
+func (b *Builder) FuncCall(f *schema.Func, args ...string) *Builder {
+	b.Func(f).rewriteLastByte('(')
+	b.MapComma(args, func(i int, b *Builder) {
+		b.WriteString(args[i])
+	})
+	b.WriteByte(')')
+	return b
+}
+
 // Proc writes the procedure identifier to the builder, prefixed
 // with the schema name if exists.
 func (b *Builder) Proc(p *schema.Proc) *Builder {
 	return b.mayQualify(p.Schema, p.Name)
 }
 
-// TableResource writes the table's resource identifier to the builder, prefixed
+// ProcCall writes the procedure identifier to the builder as a procedure call,
+func (b *Builder) ProcCall(p *schema.Proc, args ...string) *Builder {
+	b.Proc(p).rewriteLastByte('(')
+	b.MapComma(args, func(i int, b *Builder) {
+		b.WriteString(args[i])
+	})
+	b.WriteByte(')')
+	return b
+}
+
+// TableResource writes the table resource identifier to the builder, prefixed
 // with the schema name if exists.
 func (b *Builder) TableResource(t *schema.Table, r any) *Builder {
 	switch c := r.(type) {
 	case *schema.Column:
-		return b.mayQualify(t.Schema, t.Name, c.Name)
+		return b.TableColumn(t, c)
 	case *schema.Index:
 		return b.mayQualify(t.Schema, t.Name, c.Name)
 	default:
@@ -313,6 +346,12 @@ func (b *Builder) TableResource(t *schema.Table, r any) *Builder {
 	}
 }
 
+// SchemaResource writes the schema resource identifier to the builder, prefixed
+// with the schema name if exists.
+func (b *Builder) SchemaResource(s *schema.Schema, name string) *Builder {
+	return b.mayQualify(s, name)
+}
+
 func (b *Builder) mayQualify(s *schema.Schema, top string, children ...string) *Builder {
 	switch {
 	// Custom qualifier.
@@ -441,6 +480,13 @@ func (b *Builder) Wrap(f func(b *Builder)) *Builder {
 	return b
 }
 
+// WrapErr wraps the written string with parentheses
+func (b *Builder) WrapErr(f func(b *Builder) error) error {
+	var err error
+	b.Wrap(func(b *Builder) { err = f(b) })
+	return err
+}
+
 // WrapIndent is like Wrap but with extra level of indentation.
 func (b *Builder) WrapIndent(f func(b *Builder)) *Builder {
 	return b.Wrap(func(b *Builder) {
