@@ -219,12 +219,20 @@ func (d *diff) Normalize(from, to *schema.Table) error {
 	}
 	from.Indexes = indexes
 
-	// Avoid proposing changes to the table COLLATE or CHARSET
-	// in case only one of these properties is defined.
-	if err := d.defaultCollate(&to.Attrs); err != nil {
-		return err
+	// In case the "current" state was inspected (or loaded) with the collation/charset attributes,
+	// but there are not found on the desired state, detect what are the default settings for the
+	// desired state of the table (based on database default) to avoid proposing unnecessary changes.
+	if sqlx.Has(from.Attrs, &schema.Collation{}) {
+		if err := d.defaultCollate(&to.Attrs); err != nil {
+			return err
+		}
 	}
-	return d.defaultCharset(&to.Attrs)
+	if sqlx.Has(from.Attrs, &schema.Charset{}) {
+		if err := d.defaultCharset(&to.Attrs); err != nil {
+			return err
+		}
+	}
+	return nil
 }
 
 // FindTable implements the DiffDriver.TableFinder method in order to provide
