@@ -146,15 +146,15 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		if err := specutil.Scan(v, d.ScanDoc(), scanFuncs); err != nil {
 			return fmt.Errorf("specutil: failed converting to *schema.Realm: %w", err)
 		}
-		if err := convertEnums(d.Tables, d.Enums, v); err != nil {
-			return err
-		}
-		if err := convertDomains(d.Tables, d.Domains, v); err != nil {
+		if err := convertTypes(&d, v); err != nil {
 			return err
 		}
 		if err := convertSequences(d.Tables, d.Sequences, v); err != nil {
 			return err
 		}
+		if err := normalizeRealm(v); err != nil {
+			return err
+		}
 	case *schema.Schema:
 		var d doc
 		if err := hclState.Eval(p, &d, input); err != nil {
@@ -167,15 +167,15 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		if err := specutil.Scan(r, d.ScanDoc(), scanFuncs); err != nil {
 			return err
 		}
-		if err := convertEnums(d.Tables, d.Enums, r); err != nil {
-			return err
-		}
-		if err := convertDomains(d.Tables, d.Domains, r); err != nil {
+		if err := convertTypes(&d, r); err != nil {
 			return err
 		}
 		if err := convertSequences(d.Tables, d.Sequences, r); err != nil {
 			return err
 		}
+		if err := normalizeRealm(r); err != nil {
+			return err
+		}
 		*v = *r.Schemas[0]
 	case schema.Schema, schema.Realm:
 		return fmt.Errorf("postgres: Eval expects a pointer: received %[1]T, expected *%[1]T", v)
@@ -585,78 +585,6 @@ func convertColumnType(spec *sqlspec.Column) (schema.Type, error) {
 	return typ, nil
 }
 
-// convertEnums converts possibly referenced column types (like enums) to
-// an actual schema.Type and sets it on the correct schema.Column.
-func convertEnums(tables []*sqlspec.Table, enums []*enum, r *schema.Realm) error {
-	if len(enums) == 0 {
-		return nil
-	}
-	byName := make(map[string]*schema.EnumType)
-	for _, e := range enums {
-		if byName[e.Name] != nil {
-			return fmt.Errorf("duplicate enum %q", e.Name)
-		}
-		ns, err := specutil.SchemaName(e.Schema)
-		if err != nil {
-			return fmt.Errorf("extract schema name from enum reference: %w", err)
-		}
-		es, ok := r.Schema(ns)
-		if !ok {
-			return fmt.Errorf("schema %q defined on enum %q was not found in realm", ns, e.Name)
-		}
-		e1 := &schema.EnumType{T: e.Name, Schema: es, Values: e.Values}
-		es.AddObjects(e1)
-		byName[e.Name] = e1
-	}
-	for _, t := range tables {
-		for _, c := range t.Columns {
-			var enum *schema.EnumType
-			switch {
-			case c.Type.IsRefTo("enum"):
-				n, err := enumName(c.Type)
-				if err != nil {
-					return err
-				}
-				e, ok := byName[n]
-				if !ok {
-					return fmt.Errorf("enum %q was not found in realm", n)
-				}
-				enum = e
-			default:
-				if n, ok := arrayType(c.Type.T); ok {
-					enum = byName[n]
-				}
-			}
-			if enum == nil {
-				continue
-			}
-			schemaT, err := specutil.SchemaName(t.Schema)
-			if err != nil {
-				return fmt.Errorf("extract schema name from table reference: %w", err)
-			}
-			ts, ok := r.Schema(schemaT)
-			if !ok {
-				return fmt.Errorf("schema %q not found in realm for table %q", schemaT, t.Name)
-			}
-			tt, ok := ts.Table(t.Name)
-			if !ok {
-				return fmt.Errorf("table %q not found in schema %q", t.Name, ts.Name)
-			}
-			cc, ok := tt.Column(c.Name)
-			if !ok {
-				return fmt.Errorf("column %q not found in table %q", c.Name, t.Name)
-			}
-			switch t := cc.Type.Type.(type) {
-			case *ArrayType:
-				t.Type = enum
-			default:
-				cc.Type.Type = enum
-			}
-		}
-	}
-	return nil
-}
-
 // enumName extracts the name of the referenced Enum from the reference string.
 func enumName(ref *schemahcl.Type) (string, error) {
 	s := strings.Split(ref.T, "$enum.")
@@ -666,13 +594,6 @@ func enumName(ref *schemahcl.Type) (string, error) {
 	return s[1], nil
 }
 
-// enumRef returns a reference string to the given enum name.
-func enumRef(n string) *schemahcl.Ref {
-	return &schemahcl.Ref{
-		V: "$enum." + n,
-	}
-}
-
 // schemaSpec converts from a concrete Postgres schema to Atlas specification.
 func schemaSpec(s *schema.Schema) (*doc, []*schema.Trigger, error) {
 	spec, err := specutil.FromSchema(s, specFuncs)
@@ -824,18 +745,19 @@ func fromIdentity(i *Identity) *schemahcl.Resource {
 
 // columnTypeSpec converts from a concrete Postgres schema.Type into sqlspec.Column Type.
 func columnTypeSpec(t schema.Type) (*sqlspec.Column, error) {
-	// Handle postgres enum types. They cannot be put into the TypeRegistry since their name is dynamic.
 	switch o := t.(type) {
 	case *schema.EnumType:
-		return &sqlspec.Column{Type: &schemahcl.Type{
-			T:     enumRef(o.T).V,
-			IsRef: true,
-		}}, nil
+		return &sqlspec.Column{
+			Type: &schemahcl.Type{
+				IsRef: true,
+				T:     specutil.ObjectRef(o.Schema, o).V},
+		}, nil
 	case *DomainType:
-		return &sqlspec.Column{Type: &schemahcl.Type{
-			T:     o.Ref().V,
-			IsRef: true,
-		}}, nil
+		return &sqlspec.Column{
+			Type: &schemahcl.Type{
+				IsRef: true,
+				T:     specutil.ObjectRef(o.Schema, o).V},
+		}, nil
 	default:
 		st, err := TypeRegistry.Convert(t)
 		if err != nil {
